#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2010, miya All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
#
#* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#* Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys, os, struct, re, codecs, commands, time, subprocess, md5

def do_enc():
  # 一時ファイルを保存するディレクトリ
  tmp_dir = '/tmp/'
  # コマンドライン引数を取得
  if len(sys.argv) < 4:
    print 'usage: ' + sys.argv[0] + ' encode_setting_name input_filename output_filename'
    print '  encode_setting_name: p1920, p1440, i1440, p1280, i720, p640, iphone, psp'
    sys.exit()
  my_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
  enc_profile = sys.argv[1]
  full_path_r = os.path.abspath(sys.argv[2])
  full_path_w = os.path.abspath(sys.argv[3])

  # プロファイル別パラメーター
  # 書式： 'プロファイル名': ['エンコードパラメーターファイル名', 'デインターレースとスケーリングの設定', 'オーディオビットレート', 'オーディオサンプリング周波数'],
  params = {
    'p1920': ['param_h264_1920_p.txt', '-deinterlace -s 1920x1080', '160', '48000'],
    'p1440': ['param_h264_1440_p.txt', '-deinterlace -s 1440x1080', '160', '48000'],
    'i1440': ['param_h264_1440_i.txt', '-s 1440x1080', '160', '48000'],
    'p1280': ['param_h264_1280_p.txt', '-deinterlace -s 1280x720', '160', '48000'],
    'i720': ['param_h264_720_i.txt', '-s 720x480', '160', '48000'],
    'p640': ['param_h264_640_p.txt', '-deinterlace -s 640x480', '160', '48000'],
    'iphone': ['param_h264_iphone.txt', '-deinterlace -s 480x272', '160', '48000'],
    'psp': ['param_h264_psp.txt', '-deinterlace -s 480x272', '128', '44100'],
    }

  if params.has_key(enc_profile):
    option_param_file = my_dir + '/' + params[enc_profile][0]
    option_size = params[enc_profile][1]
    option_aac = '-b ' + params[enc_profile][2] + ' --no-tns --no-midside'
    option_decode_out = '-sws_flags bicubic -ar ' + params[enc_profile][3] + ' -ac 2 -pix_fmt yuv420p'
  else:
    print 'error:format'
    sys.exit()

  option_decode_in = ''
  option_mux_in = ''
  option_mux_out = ''

  # SpursEngine SDKのインストールパス
  spurs_top = os.environ['HOME'] + '/SpursEngine_Linux_SDK_v1_5_3_2/release'
  spurs_encoder = spurs_top + '/sample/release/encoder'
  dir_w = os.path.dirname(full_path_w)
  name_wt = os.path.splitext(os.path.basename(full_path_w))[0]
  name_w = 'tmp_' + hex(hash(name_wt) + 0x80000000L)
  tmp_video = tmp_dir + name_w + '_tmp.h264'
  tmp_audio = tmp_dir + name_w + '_tmp.m4a'

  # 入力ファイルにアクセス可能か
  if not os.access(full_path_r, os.R_OK):
    print 'error:read file'
    sys.exit()

  # ライブラリのパスをセット
  os.environ['LD_LIBRARY_PATH'] = spurs_top + '/linux/lib'

  # 名前付きパイプを作成
  pipe_v1 = tmp_dir + name_w + '_v1.pipe'
  pipe_a1 = tmp_dir + name_w + '_a1.pipe'
  try:
    os.mkfifo(pipe_v1)
    os.mkfifo(pipe_a1)
  except:
    print 'error:mkfifo'

  # コマンド実行
  # デコード
  process1 = subprocess.Popen(['ffmpeg -y ' + option_decode_in + ' -i "' + full_path_r + '" ' + option_size + ' ' + option_decode_out + ' -f rawvideo ' + pipe_v1 + ' -acodec pcm_s16le -f wav ' + pipe_a1], shell=True)
  # 音声エンコード
  process2 = subprocess.Popen(['faac ' + option_aac + ' -o ' + tmp_audio + ' ' + pipe_a1 + ' >/dev/null 2>&1'], shell=True)
  # 映像エンコード
  process3 = subprocess.Popen([spurs_encoder + ' ' + pipe_v1 + ' ' + tmp_video + ' 1 ' + option_param_file + ' >/dev/null 2>&1'], shell=True)
  process3.wait()
  process2.wait()
  process1.wait()
  # Mux
  process4 = subprocess.Popen(['ffmpeg -y ' + option_mux_in + ' -i ' + tmp_video + ' -i ' + tmp_audio + ' -vcodec copy -acodec copy -f mp4 ' + option_mux_out + ' "' + full_path_w + '"'], shell=True)
  process4.wait()

  # 一時ファイルを削除
  os.unlink(pipe_v1)
  os.unlink(pipe_a1)
  os.unlink(tmp_video)
  os.unlink(tmp_audio)


do_enc()
sys.exit()
